%% Bifurcation in defocusing LLE
%%
close all; format compact; keep pphome; % clean up 
plotten = 1; % save figure

if not(isfolder('start'))
    mkdir('start')
end

copyfile ../start/pt169.mat start\pt169.mat

%% Initialisation

u0 = loadp('start','pt169');
d=u0.u(u0.nu+1);zeta=u0.u(u0.nu+2); f=u0.u(u0.nu+3); epsilon=0;
p=[]; par=[d; zeta; f; epsilon];  % Parameter: d, zeta, f_0, epsilon
nx = u0.np-1; p=cLLinit(p,-pi,pi,2*nx,par); p=setfn(p,'tr'); 
u0 = [u0.u(1:nx+1);flip(u0.u(2:nx));
    u0.u(nx+2:2*(nx+1));flip(u0.u(nx+3:2*(nx+1)-1))];
p.u=[u0; par];
p.sol.ds=0.01;

%% Zeros of the effective Potential

L = sGjac(p,p.u); % Linearization
[du,eigval1] = eigs(full(L),1,'smallestabs');
[us,eigval2] = eigs(full(L'),1,'smallestabs'); % kernels of L and L*
n = p.nu/2; h = 2*pi/(p.np-1); x = getpte(p); x = x(1:end-1)';
du = du(1:n) + 1i*du(n+1:end); us = us(1:n) + 1i*us(n+1:end);
us = us/(h*real((1i*us'*conj(du))));
effV = @(s) h*real(1i*((V(x+s).*du)'*conj(us))); 
s1= fzero(effV,pi/2); s2= fzero(effV,-pi/2);

%% Shift Solition

p.nc.tol=1e-4;
p=cont(p,5);
p.sol.ds=0.05;
p.nc.tol=1e-8;
p=cont(p,160);


%% Switch to bifurcation branch and start continuation

%continue with positive ds
p=swibra('tr','bpt1','loop',0.01);
p.nc.dsmax=0.01;
p.nc.tol=1e-8;
p.sol.xi=1/p.nu^3;
p=cont(p,10);

p.sol.ds=0.01;
p.sol.xi=1/p.nu;
p = cont(p,3000);


%% compute spectrum of solitons

p = loadp('tr','pt0');
r = sGjac(p,p.u);
J = kron([0 1; -1 0], eye(p.nu/2));
stab = p.mat.M\J*r;
spec0 = eig(full(stab));

p = loadp('loop','pt20');
r = sGjac(p,p.u);
stab = p.mat.M\J*r;
spec1 = eig(full(stab));

p = loadp('loop','pt3004');
r = sGjac(p,p.u);
stab = p.mat.M\J*r;
spec2 = eig(full(stab));

%% Tracking of critical eigenvalue

p = loadp('loop','pt26');
p.file.smod=1;
p=setfn(p,'tracking_lambda');
p.sol.ds=-0.005;
p.nc.dsmax=0.005;
p.sol.xi=1/p.nu;
p = cont(p,45);

for j = 1:45
    jj = num2str(j+26);
    p = loadp('tracking_lambda',append('pt',jj));
    r = sGjac(p,p.u);
    J = kron([0 1; -1 0], eye(p.nu/2));
    stab = p.mat.M\J*r;
    lambda0(j)= eigs(stab,1,'smallestabs');
    eps_vec(j) = p.u(p.nu+4);
end

%% Generate Plots

%% Shifted Solitons and potential
figure40 = figure(40);figure40.WindowState = 'maximized';clf;
plot(x, V(x'),'LineWidth',3,'LineStyle','-','Color',[0 0 0]); hold on;
plot(x, 0*V(x'),'LineWidth',2,'LineStyle','-','Color',[0 0 0]);
p = loadp('tr','bpt1');
plot(x,0.25*(p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2),'-','Color',[0 0 1],'LineWidth',3);
plot(s2,0,'.','MarkerSize',40,'Color',[0 0 1],'LineWidth',3);
set(gca,'FontSize', 40,'XTick',[-pi -pi/2 0 pi/2 pi],'XTickLabel',{'$-\pi$','$-\frac{\pi}{2}$','$0$','$\frac{\pi}{2}$','$\pi$'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$x$','Interpreter', 'latex');
annotation(figure40,'textbox',...
    [0.467708333333333 0.751855661378347 0.106674699938306 0.0768431963578935],...
    'Color',[0 0 1],...
    'String',{'$\frac{1}{4} \times |u|^2$'},...
    'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40,...
    'EdgeColor',[1 1 1]);
xlim([-pi,pi]);
ylim([-0.5,1]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'shift_neg_blau.png');
end


figure50 = figure(50);figure50.WindowState = 'maximized';clf;
plot(x, V(x'),'LineWidth',3,'LineStyle','-','Color',[0 0 0]); hold on;
plot(x, 0*V(x'),'LineWidth',2,'LineStyle','-','Color',[0 0 0]);
p = loadp('tr','bpt2');
plot(x,0.25*(p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2),'-','Color',[0 1 0],'LineWidth',3);
plot(s1,0,'.','MarkerSize',40,'Color',[0 1 0],'LineWidth',3);
set(gca,'FontSize', 40,'XTick',[-pi -pi/2 0 pi/2 pi],'XTickLabel',{'$-\pi$','$-\frac{\pi}{2}$','$0$','$\frac{\pi}{2}$','$\pi$'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$x$','Interpreter', 'latex');
annotation(figure50,'textbox',...
    [0.467708333333333 0.751855661378347 0.106674699938306 0.0768431963578935],...
    'Color',[0 1 0],...
    'String',{'$\frac{1}{4} \times |u|^2$'},...
    'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40,...
    'EdgeColor',[1 1 1]);
xlim([-pi,pi]);
ylim([-0.5,1]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'shift_neg_green.png');
end

%% Plots of spectra

figure60=figure(60);figure60.WindowState = 'maximized';
plot(zeros(1,40000),linspace(-20000,20000,40000),'LineWidth',2,'LineStyle','-','Color',[0 0 0]); hold on;
plot(spec0,'.','Color',[0, 0 1],'MarkerSize',24);
set(gca,'FontSize', 40,'YTick',[-50 -25 0 25 50],'YTickLabel',{'-50','-25','0','25','50'});
set(gca,'TickLabelInterpreter','latex');
xlabel('Re$(\lambda)$','Interpreter', 'latex');
ylabel('Im$(\lambda)$','Interpreter', 'latex');
xlim([-2.1,0.1]);
ylim([-50,50]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'spec_neg_os.png');
end



figure70=figure(70);figure70.WindowState = 'maximized';
plot(zeros(1,40000),linspace(-20000,20000,40000),'LineWidth',2,'LineStyle','-','Color',[0 0 0]); hold on;
plot(spec1,'.','Color',[0, 0 1],'MarkerSize',24);
set(gca,'FontSize', 40,'YTick',[-50 -25 0 25 50],'YTickLabel',{'-50','-25','0','25','50'});
set(gca,'TickLabelInterpreter','latex');
xlabel('Re$(\lambda)$','Interpreter', 'latex');
ylabel('Im$(\lambda)$','Interpreter', 'latex');
xlim([-2.1,0.1]);
ylim([-50,50]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'spec_neg_stab.png');
end

figure80=figure(80);figure80.WindowState = 'maximized';
plot(zeros(1,40000),linspace(-20000,20000,40000),'LineWidth',2,'LineStyle','-','Color',[0 0 0]); hold on;
plot(spec2,'.','Color',[0, 0 1],'MarkerSize',24);
set(gca,'FontSize', 40,'YTick',[-50 -25 0 25 50],'YTickLabel',{'-50','-25','0','25','50'});
set(gca,'TickLabelInterpreter','latex');
xlabel('Re$(\lambda)$','Interpreter', 'latex');
ylabel('Im$(\lambda)$','Interpreter', 'latex');
xlim([-2.1,0.1]);
ylim([-50,50]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'spec_neg_unstab.png');
end

%% Global stability analysis

if not (exist('speicher.mat'))
    J = kron([0 1; -1 0], eye(p.nu/2));
    speicher = [];
    for j = 0:1504 %1504
        jj = num2str(2*j);
        p = loadp('loop',append('pt',jj));
        stab = p.mat.M\J*sGjac(p,p.u);
        spec = eig(full(stab));
        maxi = max(real(spec),[],'all');
        jj = num2str(2*(j+1));
        p = loadp('loop',append('pt',jj));
        if maxi > 10e-5
            speicher(j+1) = 0;
        else
            speicher(j+1) = 1;
        end
    end
else
    load("speicher.mat")
end

%% Global Bifurcation, stability changes are encoded in speicher1/2

figure100=figure(100);figure100.WindowState = 'maximized';clf;
hold on;
box on
ax = gca;
ax.BoxStyle = 'full';
p = loadp('loop','pt3010');
plot(p.branch(4,1:end),p.branch(12,1:end),'--','Color',[0, 0, 0],'LineWidth',3);
for i = 1:length(speicher)-1
    if speicher(i)
        plot(p.branch(4,2*i:2*(i+1)),p.branch(12,2*i:2*(i+1)),'-','Color',[0, 0, 0],'LineWidth',3);
    end
end
p = loadp('loop','bpt6');
plot(p.branch(4,end),p.branch(12,end),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
set(gca,'FontSize', 40,'XTick',[-2 -1 0 1 2],'XTickLabel',{'-2','-1','0','1','2'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$\varepsilon$','Interpreter', 'latex');
ylabel('$\|u\|_2^2$','Interpreter', 'latex');
xlim([-2,2]);
ylim([10.75,22.5]);
pbaspect([4 3 1]);
annotation(figure100,'textarrow',[0.5171875 0.5171875],...
    [0.790277258566978 0.750778816199377],'String',{'BP'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
hold off;
if plotten
    exportgraphics(gca,'loop_global_stab_gestrichelt.png');
end

%% Local Bifurcation
figure110 = figure(110);figure110.WindowState = 'maximized';clf;
hold on;
box on
ax = gca;
ax.BoxStyle = 'full';
p = loadp('loop','pt3010');
plot(p.branch(4,1:652*2),p.branch(12,1:652*2),'--','Color',[0 0 1],'LineWidth',3);
for i = 1:650
    if speicher(i)
        plot(p.branch(4,2*i:2*(i+1)),p.branch(12,2*i:2*(i+1)),'-','Color',[0, 0, 1],'LineWidth',3);
    end
end
plot(p.branch(4,652*2:2*1403),p.branch(12,652*2:2*1403),'--','Color',[0 1 0],'LineWidth',3);
for i = 652:1402
    if speicher(i)
        plot(p.branch(4,2*i:2*(i+1)),p.branch(12,2*i:2*(i+1)),'-','Color',[0, 1, 0],'LineWidth',3);
    end
end
plot(p.branch(4,1403*2:2*1505),p.branch(12,1403*2:2*1505),'--','Color',[0 0 1],'LineWidth',3);
for i = 1403:1504
    if speicher(i)
        plot(p.branch(4,2*i:2*(i+1)),p.branch(12,2*i:2*(i+1)),'-','Color',[0, 0, 1],'LineWidth',3);
    end
end
p = loadp('loop','bpt6');
plot(p.branch(4,end),p.branch(12,end),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
p = loadp('loop','bpt1');
plot(p.branch(4,end),p.branch(12,end),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
p = loadp('loop','bpt5');
plot(p.branch(4,end),p.branch(12,end),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
p = loadp('loop','pt64');
plot(p.branch(4,end),p.branch(12,end),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
p = loadp('loop','pt210');
plot(p.branch(4,end),p.branch(12,end),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
set(gca,'FontSize', 40,'XTick',[-0.5 0 0.5],'XTickLabel',{'-0.5','0','0.5'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$\varepsilon$','Interpreter', 'latex');
ylabel('$\|u\|_2^2$','Interpreter', 'latex');
xlim([-0.7,0.7]);
ylim([18.5,22.5]);
pbaspect([4 3 1]);
annotation(figure110,'textarrow',[0.517708333333333 0.517708333333333],...
    [0.441367601246106 0.397715472481828],'String',{'BP'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
annotation(figure110,'textarrow',[0.518229166666667 0.517708333333333],...
    [0.771585669781931 0.826583592938733],'String',{'C'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
annotation(figure110,'textarrow',[0.5546875 0.522916666666667],...
    [0.266874350986501 0.251298026998962],'String',{'A'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
annotation(figure110,'textarrow',[0.670833333333333 0.694791666666666],...
    [0.610630321910696 0.579439252336449],'String',{'B'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
annotation(figure110,'textarrow',[0.341666666666667 0.349999999999999],...
    [0.827622014537902 0.866043613707165],'String',{'D'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
hold off;
if plotten
    exportgraphics(gca,'loop_local_stab_gestrichelt.png');
end


%% Eigenvalue tracking and approximation

f = @(t) 0.5*sin(s2)*t;

figure130 = figure(130); figure130.WindowState = 'maximized';clf;
hold on;
box on
ax = gca;
ax.BoxStyle = 'full';
plot(eps_vec,lambda0,'Color',[0 0 0],'LineWidth',3,'DisplayName','$\lambda_0(\varepsilon)$');
plot(linspace(-2,2,40000),zeros(1,40000),'LineWidth',2,'LineStyle','-','Color',[0 0 0],'HandleVisibility','off');
plot(linspace(-2,2,40000),f(linspace(-2,2,40000)),'LineWidth',2,'LineStyle','--','Color',[0 0 1],'DisplayName','$-V^\prime(\sigma_0)\varepsilon$');
set(gca,'TickLabelInterpreter','latex','FontSize', 40);
xlabel('$\varepsilon$','Interpreter', 'latex');
xlim([-0.02,0.02]);
ylim([-0.02,0.02]);
lg = legend;
lg.Interpreter = 'latex';
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'neg_tracking.png');
end
%% Solitons along Branch
figure140 =figure(140);figure140.WindowState = 'maximized';
figure(140);clf;
hold on;
box on
ax = gca;
ax.BoxStyle = 'full';
p = loadp('loop','bpt5');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[1 0 0],'LineWidth',3,'DisplayName','A');
p = loadp('tr','bpt1');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[0 1 0],'LineWidth',3,'DisplayName','BP');
p = loadp('loop','pt64');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[0 0 1],'LineWidth',3,'DisplayName','B');
p = loadp('loop','bpt1');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[0 1 1],'LineWidth',3,'DisplayName','C');
p = loadp('loop','pt210');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[1 0 1],'LineWidth',3,'DisplayName','D');
set(gca,'FontSize', 40,'XTick',[-pi 0 pi],'XTickLabel',{'$-\pi$','$0$','$\pi$'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$x$','Interpreter', 'latex');
ylabel('$|u|^2$','Interpreter', 'latex');
lg = legend;
lg.Interpreter = 'latex';
lg.Location = 'southeast';
xlim([-pi,pi]);
ylim([0,5]);
pbaspect([4 3 1]);
hold off;
legend
if plotten
    exportgraphics(gca,'all_solitons_neg.png');
end

%% create/check for dir and safe files
cd ..
if not(isfolder('figures'))
    mkdir('figures')
end

cd ./d_neg_global_stab

if plotten
    movefile all_solitons_neg.png ../figures/
    movefile neg_tracking.png ../figures/
    movefile loop_local_stab_gestrichelt.png ../figures/
    movefile loop_global_stab_gestrichelt.png ../figures/
    movefile spec_neg_unstab.png ../figures/
    movefile spec_neg_stab.png ../figures/
    movefile spec_neg_os.png ../figures/
    movefile shift_neg_green.png ../figures/
    movefile shift_neg_blau.png ../figures/
end