%% Bifrucation in focusing LLE
%%
close all; format compact; keep pphome; % clean up
plotten = 1; % save figures

if not(isfolder('start'))
    mkdir('start')
end

copyfile ../start/pt290.mat start\pt290.mat
%% Initialisation

u0 = loadp('start','pt290');
d=u0.u(u0.nu+1);zeta=u0.u(u0.nu+2); f=u0.u(u0.nu+3); epsilon=0;
p=[]; par=[d; zeta; f; epsilon];  % Parameter: d, zeta, f_0, epsilon
nx = u0.np-1; p=cLLinit(p,-pi,pi,2*nx,par); p=setfn(p,'tr'); 
u0 = [flip(u0.u(2:nx));u0.u(1:nx+1);
    flip(u0.u(nx+3:2*(nx+1)-1));u0.u(nx+2:2*(nx+1))];
p.u=[u0; par];
p.sol.ds=0.01;

%% Zeros of the effective Potential

L = sGjac(p,p.u); % Linearization
[du,eigval1] = eigs(full(L),1,'smallestabs');
[us,eigval2] = eigs(full(L'),1,'smallestabs');
n = p.nu/2; h = 2*pi/(p.np-1); x = getpte(p); x = x(1:end-1)';
du = du(1:n) + 1i*du(n+1:end); us = us(1:n) + 1i*us(n+1:end);
us = us/(h*real((1i*us'*conj(du))));
effV = @(s) h*real(1i*((V(x+s).*du)'*conj(us))); 
s1= fzero(effV,pi/2); s2= fzero(effV,-pi/2);

%% Shift Solition

p.nc.tol=1e-4;
p=cont(p,5);
p.sol.ds=0.05;
p.nc.tol=1e-8;
p=cont(p,245);


%% Switch to bifurcation branch and start continuation
%% Bifurcation point 1

% continue with positive ds
p=swibra('tr','bpt1','b1+',0.01);
p.nc.dsmax=0.005;
p.nc.tol=1e-5;
p.sol.xi=1/p.nu^2;
% p.nc.dsmax=0.01; p.nc.dsmin=-0.01;
p=cont(p,5);

p.sol.ds=0.005;
p.sol.xi=1/p.nu;
p.nc.tol=1e-8;
p.nc.dsmax=0.005;
p.nc.lammax=100; p.nc.lammin=-100;p.usrlam = -0.5;
p=cont(p,220);

% continue with negative ds
p = loadp('b1+','pt5');
p=setfn(p,'b1-');
p.sol.ds=-0.005;
p.sol.xi=1/p.nu;
p.nc.tol=1e-8;
p.nc.dsmax=0.01;
p.nc.lammax=100; p.nc.lammin=-100; p.usrlam = [-0.5,0.15];
p=cont(p,170);

%% Bifurcation point 2

% continue with negative ds
p=swibra('tr','bpt2','b2-',-0.01);
p.nc.dsmax=0.005;
p.nc.tol=1e-5;
p.sol.xi=1/p.nu^2;
% p.nc.dsmax=0.01; p.nc.dsmin=-0.01;
p=cont(p,5);

p.sol.ds=0.005;
p.sol.xi=1/p.nu;
p.nc.tol=1e-8;
p.nc.dsmax=0.005;
p.nc.lammax=100; p.nc.lammin=-100;
p=cont(p,350);

% continue with positive ds
p = loadp('b2-','pt5');
p=setfn(p,'b2+');
p.sol.ds=-0.005;
p.sol.xi=1/p.nu;
p.nc.tol=1e-8;
p.nc.dsmax=0.01;
p.nc.lammax=100; p.nc.lammin=-100;
p=cont(p,130);

%% Smooth continuation near Bifurcation point
p=swibra('b1+','pt220','stab1',-0.005);
p = cont(p,550);
p=swibra('b2-','pt350','stab2',0.005);
p = cont(p,550);

%% Compute spectrum of solutions
p = loadp('b1-','bpt1');
r = sGjac(p,p.u);
J = kron([0 1; -1 0], eye(p.nu/2));
stab = p.mat.M\J*r;
spec0 = eig(full(stab));


p = loadp('b1-','pt12');
r = sGjac(p,p.u);
J = kron([0 1; -1 0], eye(p.nu/2));
stab = p.mat.M\J*r;
spec1 = eig(full(stab));

p = loadp('b1+','pt10');
r = sGjac(p,p.u);
J = kron([0 1; -1 0], eye(p.nu/2));
stab = p.mat.M\J*r;
spec2 = eig(full(stab));

%% Tracking of critical eigenvalue

p = loadp('b1+','pt20');
p=setfn(p,'tracking_lambda');
p.sol.ds=-0.005;
p.nc.dsmax=0.005;
p.sol.xi=1/p.nu;
p = cont(p,50);

for j = 1:50
    jj = num2str(j+20);
%     p = loadp('stab1',append('pt',jj));
    p = loadp('tracking_lambda',append('pt',jj));
    r = sGjac(p,p.u);
    J = kron([0 1; -1 0], eye(p.nu/2));
    stab = p.mat.M\J*r;
    lambda0(j)= eigs(stab,1,'smallestabs');
    eps_vec(j) = p.u(p.nu+4);
end
%% Generate Plots
%% Solitons along Branch
figure90 =figure(90);figure90.WindowState = 'maximized';
figure(90);clf;
hold on;
box on
ax = gca;
ax.BoxStyle = 'full';
p = loadp('b1+','pt106');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[1 0 0],'LineWidth',3,'DisplayName','A');
p = loadp('b1+','pt0');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[0 1 0],'LineWidth',3,'DisplayName','BP');
p = loadp('b1-','pt26');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[0 0 1],'LineWidth',3,'DisplayName','B');
p = loadp('b1-','bpt2');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[0 1 1],'LineWidth',3,'DisplayName','C');
p = loadp('b1-','pt116');
plot(x,p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2,'-','Color',[1 0 1],'LineWidth',3,'DisplayName','D');
set(gca,'FontSize', 40,'XTick',[-pi 0 pi],'XTickLabel',{'$-\pi$','$0$','$\pi$'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$x$','Interpreter', 'latex');
ylabel('$|u|^2$','Interpreter', 'latex');
lg = legend;
lg.Interpreter = 'latex';
xlim([-pi,pi]);
ylim([0,11]);
pbaspect([4 3 1]);
hold off;
legend
if plotten
    exportgraphics(gca,'all_solitons.png');
end
%% Shifted Solitons and potential
figure100 = figure(100);figure100.WindowState = 'maximized';clf;
plot(x, V(x'),'LineWidth',3,'LineStyle','-','Color',[0 0 0]); hold on;
plot(x, 0*V(x'),'LineWidth',2,'LineStyle','-','Color',[0 0 0]);
p = loadp('tr','bpt1');
plot(x,0.1*(p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2),'-','Color',[0 0 1],'LineWidth',3);
plot(s2,0,'.','MarkerSize',40,'Color',[0, 0 1],'LineWidth',3);
set(gca,'FontSize', 40,'XTick',[-pi -pi/2 0 pi/2 pi],'XTickLabel',{'$-\pi$','$-\frac{\pi}{2}$','$0$','$\frac{\pi}{2}$','$\pi$'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$x$','Interpreter', 'latex');
annotation(figure100,'textbox',...
    [0.3921875 0.795469368543456 0.106674699938306 0.0768431963578935],...
    'Color',[0 0 1],...
    'String',{'$\frac{1}{10} \times |u|^2$'},...
    'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40,...
    'EdgeColor',[1 1 1]);
xlim([-pi,pi]);
ylim([-0.5,1]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'shift_pos_blau.png');
end

figure110 = figure(110);figure110.WindowState = 'maximized';clf;
plot(x, V(x'),'LineWidth',3,'LineStyle','-','Color',[0 0 0]); hold on;
plot(x, 0*V(x'),'LineWidth',2,'LineStyle','-','Color',[0 0 0]);
p = loadp('tr','bpt2');
plot(x,0.1*(p.u(1:p.nu/2).^2+p.u(p.nu/2+1:p.nu).^2),'-','Color',[0 1 0],'LineWidth',3);
plot(s1,0,'.','MarkerSize',40,'Color',[0 1 0],'LineWidth',3);
set(gca,'FontSize', 40,'XTick',[-pi -pi/2 0 pi/2 pi],'XTickLabel',{'$-\pi$','$-\frac{\pi}{2}$','$0$','$\frac{\pi}{2}$','$\pi$'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$x$','Interpreter', 'latex');
annotation(figure110,'textbox',...
     [0.536979166666665 0.774700936560071 0.106674699938306 0.0768431963578929],...
    'Color',[0 1 0],...
    'String',{'$\frac{1}{10} \times |u|^2$'},...
    'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40,...
    'EdgeColor',[1 1 1]);
xlim([-pi,pi]);
ylim([-0.5,1]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'shift_pos_green.png');
end

%% Plots of spectra
figure120=figure(120);figure120.WindowState = 'maximized';
plot(zeros(1,40000),linspace(-20000,20000,40000),'LineWidth',2,'LineStyle','-','Color',[0 0 0]); hold on;
plot(spec0,'.','Color',[0 0 1],'MarkerSize',24);
set(gca,'FontSize', 40,'YTick',[-50 -25 0 25 50],'YTickLabel',{'-50','-25','0','25','50'});
set(gca,'TickLabelInterpreter','latex');
xlabel('Re$(\lambda)$','Interpreter', 'latex');
ylabel('Im$(\lambda)$','Interpreter', 'latex');
xlim([-2.1,0.1]);
ylim([-50,50]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'spec_pos_os.png');
end

figure130=figure(130);figure130.WindowState = 'maximized';
plot(zeros(1,40000),linspace(-20000,20000,40000),'LineWidth',2,'LineStyle','-','Color',[0 0 0]); hold on;
plot(spec1,'.','Color',[0 0 1],'MarkerSize',24);
set(gca,'FontSize', 40,'YTick',[-50 -25 0 25 50],'YTickLabel',{'-50','-25','0','25','50'});
set(gca,'TickLabelInterpreter','latex');
xlabel('Re$(\lambda)$','Interpreter', 'latex');
ylabel('Im$(\lambda)$','Interpreter', 'latex');
xlim([-2.1,0.1]);
ylim([-50,50]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'spec_pos_stab.png');
end

figure140=figure(140);figure140.WindowState = 'maximized';
plot(zeros(1,40000),linspace(-20000,20000,40000),'LineWidth',2,'LineStyle','-','Color',[0 0 0]); hold on;
plot(spec2,'.','Color',[0 0 1],'MarkerSize',24);
set(gca,'FontSize', 40,'YTick',[-50 -25 0 25 50],'YTickLabel',{'-50','-25','0','25','50'});
set(gca,'TickLabelInterpreter','latex');
xlabel('Re$(\lambda)$','Interpreter', 'latex');
ylabel('Im$(\lambda)$','Interpreter', 'latex');
xlim([-2.1,0.1]);
ylim([-50,50]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'spec_pos_unstab.png');
end

%% Global stability analysis
if not (exist('speicher1.mat') & exist('speicher2.mat'))
    J = kron([0 1; -1 0], eye(p.nu/2));
    speicher1 = [];
    speicher2 = [];
    for j = 0:549 %549
        jj = num2str(j);
        p = loadp('stab1',append('pt',jj));
        stab = p.mat.M\J*sGjac(p,p.u);
        spec = eig(full(stab));
        maxi = max(real(spec),[],'all');
        jj = num2str(j+1);
        p = loadp('stab1',append('pt',jj));
        if maxi > 10e-5
            speicher1(j+1) = 0;
        else
            speicher1(j+1) = 1;
        end
    end
    for j = 0:549 %549
        jj = num2str(j);
        p = loadp('stab2',append('pt',jj));
        stab = p.mat.M\J*sGjac(p,p.u);
        spec = eig(full(stab));
        maxi = max(real(spec),[],'all');
        jj = num2str(j+1);
        p = loadp('stab2',append('pt',jj));
        if maxi > 10e-5
            speicher2(j+1) = 0;
        else
            speicher2(j+1) = 1;
        end
    end
else
    load("speicher1.mat");
    load("speicher2.mat");
end

%% Global Bifurcation, stability changes are encoded in speicher1/2

figure160 = figure(160); figure160.WindowState = 'maximized';clf;
hold on;
box on
ax = gca;
ax.BoxStyle = 'full';
p = loadp('stab1','pt550');
plot(p.branch(4,1:end),p.branch(12,1:end),'--','Color',[0 0 1],'LineWidth',3);
for i = 1:length(speicher1)-1
    if speicher1(i)
         plot(p.branch(4,i:i+1),p.branch(12,i:i+1),'-','Color',[0 0 1],'LineWidth',3);
    end
end    
p = loadp('stab2','pt550');
plot(p.branch(4,1:end),p.branch(12,1:end),'--','Color',[0 1 0],'LineWidth',3);
for i = 1:length(speicher2)-1
    if speicher2(i)
         plot(p.branch(4,i:i+1),p.branch(12,i:i+1),'-','Color',[0 1 0],'LineWidth',3);
    end
end    
p = loadp('b1-','pt176');
plot(p.branch(4,26),p.branch(12,26),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
plot(p.branch(4,116),p.branch(12,116),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
p = loadp('b1+','pt226');
plot(p.branch(4,106),p.branch(12,106),'.','MarkerSize',40,'Color',[0 0 0],'LineWidth',3);
set(gca,'FontSize', 40,'XTick',[-1 -0.5 0 0.5 1],'XTickLabel',{'-1','-0.5','0','0.5','1'});
set(gca,'TickLabelInterpreter','latex');
xlabel('$\varepsilon$','Interpreter', 'latex');
ylabel('$\|u\|_2^2$','Interpreter', 'latex');
xlim([-1,1]);
ylim([3.815,3.89]);
annotation(figure160,'textarrow',[0.388541666666667 0.388541666666667],...
    [0.500557632398754 0.551401869158878],'String',{'A'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
annotation(figure160,'textarrow',[0.577604166666667 0.560416666666667],...
    [0.272104880581516 0.241952232606438],'String',{'B'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
annotation(figure160,'textarrow',[0.389583333333333 0.389583333333333],...
    [0.711357217030114 0.664589823468328],'String',{'D'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',40);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'bif_global_stab_gestrichelt.png');
end

%% Local Bifurcation

figure170 = figure(170); figure170.WindowState = 'maximized';
hold on;
box on
ax = gca;
ax.LineWidth = 3;
ax.BoxStyle = 'full';
p = loadp('stab1','pt550');
plot(p.branch(4,1:end),p.branch(12,1:end),'--','Color',[0 0 1],'LineWidth',3);
for i = 1:length(speicher1)-1
    if speicher1(i)
         plot(p.branch(4,i:i+1),p.branch(12,i:i+1),'-','Color',[0 0 1],'LineWidth',3);
    end
end    
p = loadp('stab2','pt550');
plot(p.branch(4,1:end),p.branch(12,1:end),'--','Color',[0 1 0],'LineWidth',3);
for i = 1:length(speicher2)-1
    if speicher2(i)
         plot(p.branch(4,i:i+1),p.branch(12,i:i+1),'-','Color',[0 1 0],'LineWidth',3);
    end
end  
p = loadp('b1-','bpt1');
plot(p.branch(4,end),p.branch(12,end),'.','MarkerSize',70,'Color',[0 0 0],'LineWidth',3);
p = loadp('b1-','bpt2');
plot(p.branch(4,end),p.branch(12,end),'.','MarkerSize',70,'Color',[0 0 0],'LineWidth',3);
set(gca,'FontSize', 40,'YTickLabel',[],'XTickLabel',[]);
set(gca,'TickLabelInterpreter','latex');
xlim([-0.05,0.05]);
ylim([3.825,3.838]);
pbaspect([4 3 1]);
annotation(figure170,'textarrow',[0.518229166666666 0.518229166666666],...
    [0.400830737279335 0.456905503634475],'String',{'BP'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',70);
annotation(figure170,'textarrow',[0.517708333333333 0.517708333333333],...
    [0.731087227414329 0.67497403946002],'String',{'C'},'LineWidth',3,...
    'Interpreter','latex',...
    'FontSize',70);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'bif_local_stab_gestrichelt.png');
end

%% Eigenvalue tracking and approximation

f = @(t) 0.5*sin(s2)*t;

figure190 = figure(190); figure190.WindowState = 'maximized';clf;
hold on;
box on
ax = gca;
ax.BoxStyle = 'full';
plot(eps_vec,lambda0,'Color',[0 0 0],'LineWidth',3,'DisplayName','$\lambda_0(\varepsilon)$');
plot(linspace(-2,2,40000),zeros(1,40000),'LineWidth',2,'LineStyle','-','Color',[0 0 0],'HandleVisibility','off');
plot(linspace(-2,2,40000),f(linspace(-2,2,40000)),'LineWidth',2,'LineStyle','--','Color',[0 0 1],'DisplayName','$-V^\prime(\sigma_0)\varepsilon$');
set(gca,'TickLabelInterpreter','latex','FontSize', 40);
xlabel('$\varepsilon$','Interpreter', 'latex');
lg = legend;
lg.Interpreter = 'latex';
xlim([-0.05,0.05]);
ylim([-0.05,0.05]);
pbaspect([4 3 1]);
hold off;
if plotten
    exportgraphics(gca,'pos_tracking.png');
end

%% create/check for dir and safe files
cd ..
if not(isfolder('figures'))
    mkdir('figures')
end

cd ./d_pos_global_stab

if plotten
    movefile all_solitons.png ../figures/
    movefile shift_pos_blau.png ../figures/
    movefile shift_pos_green.png ../figures/
    movefile spec_pos_os.png ../figures/
    movefile spec_pos_stab.png ../figures/
    movefile spec_pos_unstab.png ../figures/
    movefile bif_global_stab_gestrichelt.png ../figures/
    movefile bif_local_stab_gestrichelt.png ../figures/
    movefile pos_tracking.png ../figures/
end
